package chat;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import limone.*;

/**
 * The GUI for ChatUser, displays the rooms the user can join and
 * allows the user to create other rooms.
 *
 * @author Chien-Liang Fok
 * @version 5/13/2003
 */
public class ChatUserGUI extends JFrame
	implements ActionListener, java.io.Serializable{
	
	private TreeModel treeModel;
	
	/**
	 * The tree that displays the current people.
	 */
	private JTree tree;
	
	/**
	 * For use in anonymous classes
	 */
	private JFrame frame = this;
	
	/**
	 * The user of this gui.
	 */
	private ChatUser user;
	
	
	public ChatUserGUI(final ChatUser user) {
		this.user = user;
		initGUI();
		addWindowListener(new WindowAdapter(){
					public void windowClosing(WindowEvent e){
						log("Calling user.kill()");
						user.kill();
					}
					
					public void windowActivated(WindowEvent e){
						//chatUser.refreshLimeState();
					}
				});
		setSize(210,500);
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = this.getSize();
		if (frameSize.height > screenSize.height)
			frameSize.height = screenSize.height;
		if (frameSize.width > screenSize.width)
			frameSize.width = screenSize.width;
		setLocation(10, (screenSize.height - frameSize.height) / 2 + 100);
		show();
	}
	
	private void initGUI(){
		this.setTitle("MobiChat " + ChatUserAgent.VERSION_NUMBER + " - "
						  + user.getName() );
		getContentPane().setLayout(new BorderLayout());
		
		// construct tree
		tree = new JTree(treeModel = new TreeModel());
		tree.setEditable(false);
		tree.setRootVisible(false);
		tree.getSelectionModel().setSelectionMode(
			TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setBackground(Color.white);
		
		// this is the listener that is added to the tree
		MouseListener ml = new MouseAdapter(){
			public void mousePressed(MouseEvent e){
				final TreePath path = tree.getPathForLocation(e.getX(), e.getY());
				tree.setSelectionPath(path);
				if (path != null){
					// double left click
					if (e.getClickCount() == 2 && e.getModifiers() == InputEvent.BUTTON1_MASK) {
						Object selObj = tree.getLastSelectedPathComponent();
						if (selObj instanceof TreeLeaf) {
							TreeLeaf leaf = (TreeLeaf)selObj;
							user.startIM(leaf.getUserInfo().aID);
						}
					}
						// single right click
					else if (e.getClickCount() == 1 && e.getModifiers() == InputEvent.BUTTON3_MASK)  {
						Object selObj = tree.getLastSelectedPathComponent();
						if (selObj instanceof TreeLeaf){
							final TreeLeaf leaf = (TreeLeaf)selObj;
							// Allow user to send an instant message
							JPopupMenu rClickMenu = new JPopupMenu();
							rClickMenu.setInvoker(getContentPane());
							rClickMenu.add(createInstantMessageMI(leaf));
							rClickMenu.pack();
							rClickMenu.show(tree, e.getX(), e.getY());
						}
					}
				}
			}
		};
		
		tree.addMouseListener(ml);
		
		getContentPane().add( new JScrollPane(tree), BorderLayout.CENTER);
		
		createDropMenu();
	}
	
	private JMenuItem createInstantMessageMI(final TreeLeaf leaf){
		JMenuItem instantMessageMI = new JMenuItem("Send Instant Message");
		instantMessageMI.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent ae) {
						user.startIM(leaf.getUserInfo().aID);
					}
				});
		return instantMessageMI;
	}
	
	/**
	 * Impelmenents the Action Listener interface
	 *
	 * @param event the event that generated the event
	 */
	public void actionPerformed(ActionEvent event){
		if (event.getActionCommand().equals("Exit Program"))
			user.kill();
	}
	
	private void createDropMenu() {
		JMenuBar menuBar = new JMenuBar();
		
		// File Menu
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');
		JMenuItem exitMenuItem = new JMenuItem("Exit", KeyEvent.VK_X);
		exitMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
														   ActionEvent.CTRL_MASK));
		
		exitMenuItem.setActionCommand("Exit Program");
		exitMenuItem.addActionListener(this);
		fileMenu.add(exitMenuItem);
		menuBar.add(fileMenu);
		
		// Option Menu
		/*
		 JMenu oMenu = new JMenu("Options");
		 oMenu.setMnemonic('O');
		 
		 
		 JMenu soundMenu = new JMenu("Audio Settings");
		 soundMenu.setMnemonic(KeyEvent.VK_A);
		 
		 JMenuItem audioOn = new JRadioButtonMenuItem("On", true);
		 JMenuItem audioOff = new JRadioButtonMenuItem("Off", false);
		 audioOn.addActionListener(new ActionListener(){
		 public void actionPerformed(ActionEvent ae){
		 MobiChat.setPlaySound(true);
		 }
		 });
		 audioOff.addActionListener(new ActionListener(){
		 public void actionPerformed(ActionEvent ae){
		 MobiChat.setPlaySound(false);
		 }
		 });
		 
		 ButtonGroup radioGroupAudio = new ButtonGroup();
		 radioGroupAudio.add(audioOn);
		 radioGroupAudio.add(audioOff);
		 soundMenu.add(audioOn);
		 soundMenu.add(audioOff);
		 oMenu.add(soundMenu);
		 
		 
		 menuBar.add(oMenu);
		 */
		
		setJMenuBar(menuBar);
	}
	
	public void userEntered(UserInfo info) {
		treeModel.addLeaf("All Users", info);
		expandTreeCompletely();
	}
	
	public void userLeft(UserInfo info) {
		treeModel.removeLeaf("All Users", info);
		expandTreeCompletely();
	}
	
	public void userChanged(UserInfo info) {
		treeModel.updateLeaf("All Users", info);
		expandTreeCompletely();
	}
	
	private void expandTreeCompletely(){
		for (int i = 0; i < tree.getRowCount(); i++){
			tree.expandRow(i);
		}
	}
	
	private void log(String str){
		System.out.println("ChatUserGUI: " + str);
		System.out.flush();
	}
	
	/**
	 * Returns the string representation of this class.
	 *
	 * @return the string representation of this class.
	 */
	public String toString(){
		return "ChatUserGUI, " + user.getName();
	}
}
