package chat;

import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;

/**
 * An implementation of the root of the user tree.
 *
 * @author Chien-Liang Fok
 * @version 1/2/2002
 */
public class TreeRoot implements java.io.Serializable{
	/** 
	  * Contains TreeBranch objects keyed by category
	  */	
	private Vector branches = new Vector(); 
	
	/**
	 * Creates the root of the user tree.
	 */
	public TreeRoot(){
	}
	
	/**
	 * Returns the number of elements in the tree
	 *
	 * @param int the number of branches in this tree
	 */
	public synchronized int getNumBranches(){
		return branches.size();
	}
	
	/**
	 * Adds a leaf into the tree
	 *
	 * @param cutl the leaf to be inserted
	 * @return true if successful
	 */
	public synchronized boolean addLeaf(String category, UserInfo info, 
		TreeChangeResults results){

		log("addLeaf - calling branches.indexOf(category)");
		//if (branches.size() > 0)
		//	log("addLeaf - firstBranch is " + branches.get(0));
		int index = branches.indexOf(new TreeBranch(category));		
		TreeBranch branch;
		if (index == -1) { // the branch did not exist, create a new branch
			log("addLeaf - the branch did not already exist");
			branch = new TreeBranch(category);
			branches.add(branch);
			results.addNode(branches.indexOf(branch), branch);
			results.finalize();
		} 
		else {
			log("addLeaf - the branch already existed");
			branch = (TreeBranch)branches.get(index);
			results.addToPath(branch);
		}
		return branch.addLeaf(info, results); // add the leaf to the branch		
	}

	/**
	  * Updates a leaf's userInfo within the given category.
	  *
	  * @return true if the update was successful
	  */
	public synchronized boolean updateLeaf(String category, UserInfo info, TreeChangeResults results){		
		log("updateLeaf - calling branches.indexOf(category)");
		//if (branches.size() > 0)
		//	log("addLeaf - firstBranch is " + branches.get(0));
		int index = branches.indexOf(new TreeBranch(category));		
		TreeBranch branch;
		if (index == -1) { // the branch did not exist, create a new branch
			log("updateLeaf - the branch did not already exist");
			return false;
		} 
		else {
			log("updateLeaf - the branch already existed");
			branch = (TreeBranch)branches.get(index);
			results.addToPath(branch);
			return branch.updateLeaf(info, results); // add the leaf to the branch
		}
		
		
	}

	/**
	  * Removes a leaf from the tree.
	  *
	  * @return true if successful
	  */
	public synchronized boolean removeLeaf(String category, UserInfo info, 
		TreeChangeResults results){	
		
		int index = branches.indexOf(new TreeBranch(category));				
		if (index == -1) {
			log("The branch did not exist");
			return false;
		}
		
		TreeBranch branch;
		branch = (TreeBranch)branches.get(index);
		if (branch.getNumLeaves() == 1 
			&& branch.getIndexOfChild(new TreeLeaf(info)) != -1) {
			log("The branch will no longer have any children, removing branch");
			// remove the leaf
			branches.remove(branch);
			results.addNode(index, branch);
			return true;
		}
		else {
			log("Removing leaf from branch");
			results.addToPath(branch);
			return branch.removeLeaf(info, results);
		}
	}
	
	/** 
	 * Returns the index of the child or -1 if the child does not exist
	 *
	 * @param the child
	 * @return the index of the child
	 */
	public synchronized int getIndexOfChild(Object child){
		return branches.indexOf(child);
	}

	public synchronized Object getChild(int index){
		return branches.get(index);
	}
	
	private void log(String str){
		//System.out.println("TreeRoot: " + str);
		//System.out.flush();
	}
	
	/**
	 * Returns the String representation of this class
	 *
	 * @return the String representation of this class
	 */
	public String toString(){
		return "TreeRoot";
	}
}
